package lab.actors 
{
	import as3isolib.display.IsoSprite;
	import flash.geom.Point;
	import lab.actors.api.BaseActor;
	import lab.data.ItemVO;
	import lab.actors.api.BaseActor;
	import lab.core.IsoLevel;
	import lab.data.Items;
	import lab.data.ItemVO;
	import lab.display.DataImage;
	import lab.embed.Atlases;
	import lab.registry.Reg;
	import starling.animation.Tween;
	import starling.core.Starling;
	import starling.textures.Texture;
	/**
	 * ...
	 * @author Roma Kraminskij - kraminskij@gmail.com
	 */
	public class EnemyActor extends BaseActor 
	{
		private var isBoxOver:Boolean = false;
		private var targets:Vector.<ItemVO> = new Vector.<ItemVO>;
		private var texturesDirection:Vector.<Texture> = new Vector.<Texture>;
		private var speed:Number = 1; // 1 cell per second.
		private var direction:Point = new Point( 0, 0 );
		private var currentPosition:int = 0; // Position in targets list.
		private var image:DataImage;
		
		public function EnemyActor() 
		{
			super();
			
		}
		/**
		 * Build actors sprites and actions.
		 * @param	item
		 * @param	level
		 */
		public override function start( item:ItemVO = null, level:IsoLevel=null ) : void 
		{
			super.start(item, level);
			
			texturesDirection = Atlases.getAtlas( "tiles_00" ).getTextures( "enemy_" );
			
			var texture:Texture = Atlases.getAtlas( "tiles_00" ).getTexture( item.type );
			image = new DataImage( texture, item );
			
			var k:Number = texture.height / texture.width;
			
			image.width = IsoLevel.CELL_SIZE * 2;
			image.height = IsoLevel.CELL_SIZE * 2;
			image.x = -IsoLevel.CELL_SIZE;
			image.y = -IsoLevel.CELL_SIZE - IsoLevel.CELL_SIZE / 4;
			image.smoothing = Reg.TEXTURE_SMOOTHING;
			
			var sprite:IsoSprite = new IsoSprite();
			
			sprite.name = item.type;
			sprite.sprites = [ image ];
			sprite.data = item;
			sprite.setSize( IsoLevel.CELL_SIZE-1, IsoLevel.CELL_SIZE-1, 1);
			sprite.moveTo( item.x * IsoLevel.CELL_SIZE-1, item.y * IsoLevel.CELL_SIZE-1, 0 );
			
			level.scene.addChild( sprite );
			
			item.addObj( sprite );
			
			if ( item.source.custom[0] != null )
			{
				var speed:Number = item.source.custom[0].@speed;
				targets = getTargets( item.source.custom[0].@path );
			}
			
			move();
		}
		/**
		 * Calc direction and start animation.
		 */
		private function move() : void 
		{
			// Set new direction of move.
			if ( item.x > targets[currentPosition].x ) direction.x = -1 else direction.x = 1;
			if ( item.x == targets[currentPosition].x ) direction.x = 0;
			
			if ( item.y > targets[currentPosition].y ) direction.y = -1 else direction.y = 1;
			if ( item.y == targets[currentPosition].y ) direction.y = 0;
			
			if ( direction.y > 0 && direction.x == 0 ) image.texture = texturesDirection[0];
			if ( direction.y == 0 && direction.x > 0 ) image.texture = texturesDirection[1]
			
			if ( direction.y < 0 && direction.x == 0 ) image.texture = texturesDirection[3];
			if ( direction.y == 0 && direction.x < 0 ) image.texture = texturesDirection[2];
			
			animateEnemy();
			
			// Check position on next target, so get next target.
			if ( item.x == targets[currentPosition].x && item.y == targets[currentPosition].y )
			{
				currentPosition++;
				// Check end of targets list.
				if ( currentPosition > (targets.length - 1) ) currentPosition = 0;
			}
		}
		
		private function animateEnemy() : void
		{
			if ( !direction ) return;
			
			var tw0:Tween = new Tween( item.sprites[0], speed );
			
			tw0.animate("x", (item.x + direction.x) * IsoLevel.CELL_SIZE);
			tw0.animate("y", (item.y + direction.y) * IsoLevel.CELL_SIZE);
			
			tw0.onUpdate = updateScene;
			tw0.onComplete = complete;
			Starling.current.juggler.add(tw0);
		}
		
		private function complete():void 
		{
			// New position.
			item.x += direction.x;
			item.y += direction.y;
			// Check player.
			var p:ItemVO = level.getItemByType( Items.PLAYER );
			
			if ( p != null && p.actor != null && p.actor is PlayerActor ) 
			{
				if ( p.x == item.x && p.y == item.y ) 
				{
					level.onLevelEnd.dispatch();
					return;
				}
				
				if ( (p.actor as PlayerActor).isoXT == item.x && (p.actor as PlayerActor).isoYT == item.y ) 
				{
					level.onLevelEnd.dispatch();
					return;
				}
			}
			
			move();
		}
		
		private function updateScene():void 
		{
			level.isNeedUpdate = true;
		}
		
		public override function check():void
		{
			
		}
		/**
		 * 
		 * @param	sourcePath
		 * @return
		 */
		private function getTargets( sourcePath:String ): Vector.<ItemVO>
		{
			var v:Vector.<ItemVO> = new Vector.<ItemVO>;
			
			for each ( var s:String in sourcePath.split(",") )
			{
				var arr0:Array = s.split(":");
				v.push( level.getItemTypeOfXY( Items.FLOOR, arr0[0], arr0[1] ) );
			}
			
			return v;
		}
	}

}









































