package lab.actors 
{
	import as3isolib.display.IsoSprite;
	import lab.actors.api.BaseActor;
	import lab.core.IsoLevel;
	import lab.data.Items;
	import lab.data.ItemVO;
	import lab.display.DataImage;
	import lab.embed.Atlases;
	import lab.registry.Reg;
	import starling.animation.Tween;
	import starling.core.Starling;
	import starling.textures.Texture;
	
	/**
	 * ...
	 * @author Roma Kraminskij - kraminskij@gmail.com
	 */
	public class KeyTriggerActor extends BaseActor 
	{
		private var isPicked:Boolean = false;
		private var target:ItemVO = null;
		
		public function KeyTriggerActor() 
		{
			//super( item, level);
		}
		
		/**
		 * Build actors sprites and actions.
		 * @param	item
		 * @param	level
		 */
		public override function start( item:ItemVO = null, level:IsoLevel=null ) : void 
		{
			super.start(item, level);
			
			var texture:Texture = Atlases.getAtlas( "tiles_00" ).getTexture( item.type );
			var image:DataImage = new DataImage( texture, item );
			
			var k:Number = texture.height / texture.width;
			
			image.width = IsoLevel.CELL_SIZE * 2;
			image.height = IsoLevel.CELL_SIZE * 2 * k;
			image.x = -IsoLevel.CELL_SIZE;
			image.y = -( ( IsoLevel.CELL_SIZE * 2 * k ) - IsoLevel.CELL_SIZE );
			image.smoothing = Reg.TEXTURE_SMOOTHING;
			
			var sprite:IsoSprite = new IsoSprite();
			
			sprite.name = item.type;
			sprite.sprites = [ image ];
			sprite.data = item;
			sprite.setSize( IsoLevel.CELL_SIZE, IsoLevel.CELL_SIZE, IsoLevel.CELL_SIZE );
			sprite.moveTo( item.x * IsoLevel.CELL_SIZE-1, item.y * IsoLevel.CELL_SIZE-1, 1 );
			
			level.scene.addChild( sprite );
			
			item.addObj( sprite );
			
			var c:XML = item.source.custom[0];
			
			var x0:Number = c.@xt;
			var y0:Number = c.@yt;
			
			if ( c != null )
			{
				target = level.getItemTypeOfXY( Items.DOOR, x0, y0 );
				trace();
			}
		}
		
		public override function check():void
		{
			if ( isPicked ) return;
			// Check for setted box.
			if ( level.isItemsHasTypeOfXY( Items.PLAYER, item.x, item.y ) ) 
			{
				isPicked = true;
				animateDoor();
				animateKey();
			}
		}
		
		private function animateKey():void
		{
			var tw0:Tween = new Tween( item.sprites[0].container, 2 );
			tw0.animate( "y", IsoLevel.CELL_SIZE * 2);
			tw0.animate( "alpha", 0 );
			Starling.current.juggler.add(tw0);
		}
		
		private function animateDoor():void
		{
			var direction:int = ( target.source.custom[0].@state == "up" ) ? -IsoLevel.CELL_SIZE  : -1;
			
			var tw0:Tween = new Tween( target.sprites[0], 2 );
			tw0.animate("z", direction);
			tw0.onUpdate = updateScene;
			tw0.onComplete = complete;
			Starling.current.juggler.add(tw0);
		}
		
		private function complete():void 
		{
			target.source.custom[0].@state = ( target.source.custom[0].@state == "up" ) ? "down" : "up";
		}
		
		private function updateScene():void 
		{
			level.isNeedUpdate = true;
		}
	}

}