package mvc.ui.editor.mediators 
{
	import feathers.controls.Alert;
	import feathers.core.PopUpManager;
	import feathers.data.ListCollection;
	import flash.events.IEventDispatcher;
	import flash.events.MouseEvent;
	import flash.system.System;
	import lab.embed.Atlases;
	import mvc.ui.editor.models.EditorItemVO;
	import mvc.ui.editor.models.EditorModel;
	import mvc.ui.editor.views.EditorView;
	import mvc.ui.game.events.GameEvent;
	import mvc.ui.game.models.GameModel;
	import mvc.ui.game.models.LevelModel;
	import robotlegs.bender.extensions.starlingViewMap.impl.StarlingMediator;
	import starling.core.Starling;
	import starling.events.Event;
	
	/**
	 * ...
	 * @author Roma Kraminskij - kraminskij@gmail.com
	 */
	public class EditorViewMediator extends StarlingMediator 
	{
		[Inject] public var view:EditorView;
		[Inject] public var gameModel:GameModel;
		[Inject] public var levelModel:LevelModel;
		[Inject] public var editorModel:EditorModel;
		[Inject] public var dispatcher:IEventDispatcher;
		
		private var items:ListCollection;
		private var currentItem:EditorItemVO;
		private var alert:Alert;
		
		public function EditorViewMediator() 
		{
		}
		
		override public function initialize():void
		{
			dispatcher.addEventListener( GameEvent.LEVEL_ENDED, endLevelHandler );
			
			buildList();
			
			view.list.addEventListener(Event.CHANGE, listChangeHandler );
			// TODO: Need TouchEvent, TouchPhase.START for mouse up. Event.TRIGGERED is going after Mouse up.
			view.saveButton.addEventListener( Event.TRIGGERED, copyHandler );
			view.exitButton.addEventListener( Event.TRIGGERED, exitHandler );
		}
		
		private function exitHandler(e:Event):void 
		{
			endLevelHandler();
		}
		
		private function copyHandler(e:Event):void 
		{
			Starling.current.nativeStage.addEventListener( MouseEvent.MOUSE_UP, upHandler );
		}
		
		private function upHandler(e:MouseEvent):void 
		{
			Starling.current.nativeStage.removeEventListener( MouseEvent.MOUSE_UP, upHandler );
			var s:String = levelModel.toXml().toString();
			if ( s != null && s.length > 0 ) System.setClipboard( s );
		}
		
		private function buildList():void
		{
			var arr:Array = new Array();
			
			var list:XML = Atlases.getObjects( "tiles_00" );
			
			for each ( var x:XML in list.SubTexture )
			{
				arr.push(new EditorItemVO( x ) );
			}
			// TODO: Delete item need to push at start of list.
			/*var xml:XML = <object name='delete_00' type='delete' atlas='gui_00'/>;
			arr.push( new EditorItemVO( xml ) );*/
			items = new ListCollection(arr);
			
			view.list.dataProvider = items;
			
			editorModel.currentItem = arr[0];
		}
		
		private function listChangeHandler(e:Event):void 
		{
			currentItem = items.getItemAt( view.list.selectedIndex ) as EditorItemVO;
			trace(currentItem.name);
			editorModel.currentItem = currentItem;
		}
		
		private function endLevelHandler(e:GameEvent=null):void
		{
			if ( gameModel.isEditorMode ) 
			{
				// TODO: Ask user about exit without saving.
				//PopUpManager.centerPopUp( Alert.show( "Close map?" ) );
				//alert = Alert.show( "This is an unsaved data in level!", "This is an unsaved data in level!", 
				//			new ListCollection( [ { label: "OK" } ] ) );
				//alert.addEventListener( Event.CLOSE, closeHandler );
				//view.removeFromParent(true);
			}
		}
		
		private function closeHandler(e:Event):void 
		{
			trace();
			//PopUpManager.removePopUp( alert, true );
		}
		
	}

}


































