package mvc.ui.toolspanel.mediators 
{
	import citrus.sounds.SoundManager;
	import feathers.data.ListCollection;
	import flash.events.IEventDispatcher;
	import lab.embed.Atlases;
	import mvc.events.AppEvent;
	import mvc.ui.game.events.GameEvent;
	import mvc.ui.root.models.AppModel;
	import mvc.ui.toolspanel.views.ToolsView;
	import robotlegs.bender.extensions.starlingViewMap.impl.StarlingMediator;
	import starling.display.Image;
	import starling.events.Event;
	
	/**
	 * ...
	 * @author Roma Kraminskij - kraminskij@gmail.com
	 */
	public class ToolsViewMediator extends StarlingMediator 
	{
		[Inject] public var view:ToolsView;
		[Inject] public var appModel:AppModel;
		[Inject] public var dispatcher:IEventDispatcher;
		
		private var gameStateData:ListCollection;
		private var menuStateData:ListCollection;
		
		public function ToolsViewMediator() 
		{
		}
		
		override public function initialize():void
		{
			// Game buttons list.
			gameStateData = new ListCollection(
			[
				{ defaultIcon: getIcon( "full_in" ), eventType: GameEvent.SWITCH_FULLSCREEN, eventClass: GameEvent,
					 isToggle: true, defaultSelectedIcon: getIcon( "full_out" ) },
				{ defaultIcon: getIcon( "menu" ), eventType: AppEvent.REQUEST_MENU_VIEW, eventClass: AppEvent },
				{ defaultIcon: getIcon( "replay" ), eventType: GameEvent.LEVEL_SELECTED, eventClass: GameEvent },
				{ defaultIcon: getIcon( "sound" ), typeSound: "sound" }, 
				{ defaultIcon: getIcon( "music" ), 	typeSound: "music" },
				{ defaultIcon: getIcon( "options" ), eventType: AppEvent.REQUEST_OPTIONS_VIEW, eventClass: AppEvent }
			]);
			// Menu buttons list.
			menuStateData = new ListCollection(
			[
				{ defaultIcon: getIcon( "full_in" ), eventType: GameEvent.SWITCH_FULLSCREEN, eventClass: GameEvent,
					 isToggle: true, defaultSelectedIcon: getIcon( "full_out" ) },
				{ defaultIcon: getIcon( "sound" ), typeSound: "sound" }, 
				{ defaultIcon: getIcon( "music" ), 	typeSound: "music" },
				{ defaultIcon: getIcon( "options" ), eventType: AppEvent.REQUEST_OPTIONS_VIEW, eventClass: AppEvent }
			]);
			
			view.init( menuStateData );
			
			view.buttonGroup.addEventListener(Event.TRIGGERED, barHandler );
			
			dispatcher.addEventListener( GameEvent.LEVEL_ENDED, switchState );
			dispatcher.addEventListener( GameEvent.LEVEL_SELECTED, switchState );
		}
		
		
		private function switchState( e:GameEvent = null ):void 
		{
			switch ( e.type )
			{
				case GameEvent.LEVEL_ENDED: view.buttonGroup.dataProvider = menuStateData;
					break;
				case GameEvent.LEVEL_SELECTED: view.buttonGroup.dataProvider = gameStateData;
					break;
				default:
					break;
			}
		}
		
		private function barHandler(e:Event):void 
		{
			//var b:Button; b.defaultSelectedIcon
			
			if ( e.data.eventClass && e.data.eventType )
			{
				var ec:Class = e.data.eventClass;
				var en:String = e.data.eventType;
				dispatcher.dispatchEvent( new ec( en ) );
				return;
			}
			
			if ( e.data.typeSound ) 
			{
				if ( e.data.typeSound == "music" ) 
				{
					if ( appModel.isTrackPlaying )
					{
						SoundManager.getInstance().pauseSound( "lightlistening_loop" );
						appModel.isTrackPlaying = false;
					}
					else 
					{
						SoundManager.getInstance().playSound( "lightlistening_loop" );
						appModel.isTrackPlaying = true;
					}
				}
				
				if ( e.data.typeSound == "sound" ) 
				{
					
				}
			}
		}
		

		private function getIcon(name:String):Image
		{
			return new Image( Atlases.getAtlas( "gui_00" ).getTexture( name ) ); 
		}
	}

}



































