package swa.seq 
{
	import org.osflash.signals.Signal;
	import swa.tasks.BaseTask;
	/**
	 * ...
	 * @author Roma Kraminskij - kraminskij@gmail.com
	 */
	public class Sequence 
	{
		
		public var isLooped:Boolean = false;
		public var isBreakable:Boolean = true;
		public var tasks:Vector.<BaseTask> = new Vector.<BaseTask>;
		public var onRun:Signal = new Signal();
		public var onEnd:Signal = new Signal();
		public var onFail:Signal = new Signal();
		public var onLoop:Signal = new Signal();
		
		private var _isPaused:Boolean = false;
		private var current:int = -1;
		
		public function Sequence( isBreakable:Boolean = true ) 
		{
			this.isBreakable = isBreakable;
		}
		
		public function add( t:BaseTask = null ):void
		{
			if ( t == null ) return;
			tasks.push(t);
		}
		
		public function remove( t:BaseTask ):void
		{
			for ( var i:int = 0; i < tasks.length; i++ )
				if ( t.guid == tasks[i].guid ) 
				{
					tasks.splice( i, 1 );
					break;
				}
		}
		
		public function run():void
		{
			if ( tasks.length < 1 ) 
			{
				stop();
				return;
			}
			_isPaused = false;
			current++;
			tasks[current].onEnd.addOnce(nextTask);
			tasks[current].onFail.addOnce(fail);
			tasks[current].run();
		}
		
		public function stop():void
		{
			current = -1;
			_isPaused = true;
			if ( isLooped )
			{ 
				onLoop.dispatch();
				run();
			}
			else
			{
				onEnd.dispatch();
			}
		}
		
		public function pause():void
		{
			_isPaused = true;
		}
		
		public function get isPaused():Boolean 
		{
			return _isPaused;
		}
		
		private function nextTask():void
		{
			current++;
			
			if ( current > tasks.length - 1 ) 
			{
				stop();
			}
			else
			{
				tasks[current].onEnd.addOnce(nextTask);
				tasks[current].onFail.addOnce(fail);
				tasks[current].run();
			}
		}
		
		private function fail():void
		{
			onFail.dispatch();
		}
	}

}